#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information

"""
This is the main script for the excution of the Paineira Graphical User Interface, a GUI for visualization and data processing during in situ experiments at Paineira.
In this script, both GUIs used by the program are called and all of the backend functions and processes are defined. 
"""

import sys, time
from PyQt5.QtWidgets import QApplication, QVBoxLayout, QFileDialog, QDialog, QProgressDialog, QPushButton
from PyQt5.QtCore import Qt, QSize, QThread, pyqtSignal
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
import matplotlib.pyplot as plt
from matplotlib.widgets import SpanSelector
from matplotlib.cm import ScalarMappable
from matplotlib.backend_bases import MouseEvent
import numpy as np
import pandas as pd
from PyQt5.QtWidgets import (
    QApplication, QDialog, QMainWindow, QMessageBox
)
from Monitor import FolderMonitor
from GUI.iguape_GUI import Ui_MainWindow
from GUI.pk_window import Ui_pk_window
from Monitor import peak_fit, counter, peak_fit_split_gaussian
from PyQt5.QtCore import Qt, QThread, pyqtSignal
from PyQt5.QtWidgets import QProgressDialog, QMessageBox, QVBoxLayout

__version__ = '1.0'
license  = 'GNU GPL-3.0 License'
    
counter.count = 0

class Window(QMainWindow, Ui_MainWindow):
    """ 
    Main window Class
    """
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.create_graphs_layout()

    def create_graphs_layout(self):
        """
        Routine to create the layout for the XRD and Fitting Data Graphs
        """
        self.XRD_data_layout = QVBoxLayout()
        # Creating the main Figure and Layout #
        self.fig_main = Figure(figsize=(8, 6), dpi=100)
        self.gs_main = self.fig_main.add_gridspec(1, 1)
        self.ax_main = self.fig_main.add_subplot(self.gs_main[0, 0])
        self.fig_main.set_layout_engine('compressed')
        self.canvas_main = FigureCanvas(self.fig_main)
        self.XRD_data_layout.addWidget(self.canvas_main)
        self.XRD_data_tab.setLayout(self.XRD_data_layout)
    
        self.peak_fit_layout = QVBoxLayout()
        #Creating the fitting parameter Figure and Layout#
        self.fig_sub = Figure(figsize=(8, 5), dpi=100)
        self.gs_sub = self.fig_sub.add_gridspec(1, 3)
        self.ax_2theta = self.fig_sub.add_subplot(self.gs_sub[0, 0])
        self.ax_area = self.fig_sub.add_subplot(self.gs_sub[0, 2])
        self.ax_FWHM = self.fig_sub.add_subplot(self.gs_sub[0, 1])
        self.fig_sub.set_layout_engine('compressed')
        self.canvas_sub = FigureCanvas(self.fig_sub)
        self.peak_fit_layout.addWidget(self.canvas_sub)
        self.peak_fit_tab.setLayout(self.peak_fit_layout)

        #Creating a colormap on the main canvas#
        self.cmap = plt.get_cmap('coolwarm') 
        self.norm = plt.Normalize(vmin=0, vmax=1) # Initial placeholder values for norm #
        self.sm = ScalarMappable(cmap=self.cmap, norm=self.norm)
        self.sm.set_array([])
        self.cax = self.fig_main.colorbar(self.sm, ax=self.ax_main) # Creating the colorbar axes #
        
        #Connecting functions to buttons#
        self.refresh_button.clicked.connect(self.update_graphs)
        
        self.reset_button.clicked.connect(self.reset_interval)
        
        self.peak_fit_button.clicked.connect(self.select_fit_interval)
        self.save_peak_fit_data_button.clicked.connect(self.save_data_frame)
    
        self.plot_with_temp = False
        self.selected_interval = None
        self.fit_interval = None
        
        self.folder_selected = False
        self.temp_mask_signal = False
        self.plot_data = pd.DataFrame()

        #Create span selector on the main plot#
        self.span = SpanSelector(self.ax_main, self.onselect, 'horizontal', useblit=True,
                                props=dict(alpha=0.3, facecolor='red', capstyle='round'))

        self.XRD_data_layout.addWidget(NavigationToolbar(self.canvas_main))
        self.peak_fit_layout.addWidget(NavigationToolbar(self.canvas_sub))
        
        self.actionOpen_New_Folder.triggered.connect(self.select_folder)
        self.actionAbout.triggered.connect(self.about)
        
        self.offset_slider.setMinimum(1)
        self.offset_slider.setMaximum(100)
        self.offset_slider.setValue(10)

    
        self.XRD_measure_order_checkbox.stateChanged.connect(self.measure_order_index)
        self.temperature_checkbox.stateChanged.connect(self.temp_index)
        self.filter_button.clicked.connect(self.apply_temp_mask)

    def update_graphs(self):
        """
        Clears plotted axis, updates the colormap normalization and calls refreshing routines (_update_main_figure and _plot_fitting_parameters)
        """
        try:
            QApplication.setOverrideCursor(Qt.WaitCursor)

            self.ax_main.clear()
        
            self._update_main_figure()
            self._plot_fitting_parameters()

            self.canvas_main.draw()
            self.canvas_sub.draw()
            self.cax.update_normal(self.sm)

        except Exception as e:
            print(f'Please, initialize the monitor! Error: {e}')
            QMessageBox.warning(self, '','Please initialize the monitor!') 
            pass

        QApplication.restoreOverrideCursor()
        

    def _get_mask(self, i):
        """
        Creates a mask, based on the interval selected by the user, for the i-th XRD measure.
        If no interval is selected, returns None.
        """
        if self.selected_interval:
            return (self.plot_data['theta'][i] >= self.selected_interval[0]) & (self.plot_data['theta'][i] <= self.selected_interval[1])
        return slice(None)

    def _update_main_figure(self):
        """
        Main Figure (XRD Data) refreshing routine. It plots the XRD patterns, with the customizations selected by the user (2theta mask, temperature mask, XRD index, etc.)
        """
        try:
            self.plot_data = self.monitor.data_frame[self.temp_mask].reset_index(drop=True) if self.temp_mask_signal else self.monitor.data_frame
        except (AttributeError, pd.errors.IndexingError):
            pass
        
        def update_colormap(color_map_type, label):
            self.norm.vmin, self.norm.vmax = min(self.plot_data[color_map_type]), max(self.plot_data[color_map_type])
            self.sm.set_norm(self.norm)
            self.cax.set_label(label)

        if self.plot_with_temp:
            update_colormap('temp', 'Cryojet Temperature (K)' if self.monitor.kelvin_sginal else 'Blower Temperature (°C)')
            self.min_temp_doubleSpinBox.setRange(min(self.monitor.data_frame['temp']), max(self.monitor.data_frame['temp']))
            self.max_temp_doubleSpinBox_2.setRange(min(self.monitor.data_frame['temp']), max(self.monitor.data_frame['temp']))
            self.min_temp_doubleSpinBox.setValue(min(self.plot_data['temp']))
            self.max_temp_doubleSpinBox_2.setValue(max(self.plot_data['temp']))
        else:
            update_colormap('file_index', 'XRD measure order')
            self.min_temp_doubleSpinBox.setRange(min(self.monitor.data_frame['file_index']), max(self.monitor.data_frame['file_index']))
            self.max_temp_doubleSpinBox_2.setRange(min(self.monitor.data_frame['file_index']), max(self.monitor.data_frame['file_index']))
            self.min_temp_doubleSpinBox.setValue(min(self.plot_data['file_index']))
            self.max_temp_doubleSpinBox_2.setValue(max(self.plot_data['file_index']))
        self.spacing = max(self.plot_data['max']) / self.offset_slider.value()
        offset = 0
        for i in range(len(self.plot_data['theta'])):
            norm_col = 'temp' if self.plot_with_temp else 'file_index' #Flag for chosing the XRD pattern index
            color = self.cmap(self.norm(self.plot_data[norm_col][i])) #Selecting the pattern's color based on the colormap
            mask = self._get_mask(i)
            self.ax_main.plot(self.plot_data['theta'][i][mask], self.plot_data['intensity'][i][mask] + offset, color=color)
            offset += self.spacing

        self.ax_main.set_xlabel('2θ (°)')
        self.ax_main.set_ylabel('Intensity (u.a.)')

    def _plot_fitting_parameters(self):
        """
        Calls the fitting parameters plotting routines (_plot_single_peak or _plot_double_peak) 
        """
        if not self.fit_interval:
            return

        self.ax_2theta.clear()
        self.ax_area.clear()
        self.ax_FWHM.clear()

    
        if self.fit_interval_window.fit_model == 'PseudoVoigt':
            self._plot_single_peak()
        else:
            self._plot_double_peak()

    
        self.ax_main.axvspan(self.fit_interval[0], self.fit_interval[1], color='grey', alpha=0.5, label='Selected Fitting Interval')
        self.ax_main.legend()
        
    def _plot_single_peak(self):
        """
        Based on the x data type flag (temperature/XRD measure order), plots the fitting parameters (Peak Postion, Integrated Area and FWHM) as a function of x.
        """
        x_data_type = 'temp' if self.plot_with_temp else 'file_index'
        x_label = 'XRD measure' if not self.plot_with_temp else 'Cryojet Temperature (K)' if self.monitor.kelvin_sginal else 'Blower Temperature (°C)'
        self._plot_parameter(self.ax_2theta, self.monitor.fit_data[x_data_type], self.monitor.fit_data['dois_theta_0'], 'Peak position (°)', x_label, color='red')
        self._plot_parameter(self.ax_area, self.monitor.fit_data[x_data_type], self.monitor.fit_data['area'], 'Peak integrated area', x_label, color='green')
        self._plot_parameter(self.ax_FWHM, self.monitor.fit_data[x_data_type], self.monitor.fit_data['fwhm'], 'FWHM (°)', x_label, color='blue')

    def _plot_double_peak(self):
        """
        Based on the x data type flag (temperature/XRD measure order), plots the fitting parameters (Peak Postion, Integrated Area and FWHM) as a function of x. In this version, the fitting model is the Split Pseudo-Voigt Model (2x Pseudo-Voigt)
        """
        x_data_type = 'temp' if self.plot_with_temp else 'file_index'
        x_label = 'Cryojet Temperature (K)' if self.monitor.kelvin_sginal else 'Blower Temperature (°C)' if self.plot_with_temp else 'XRD measure'

        self._plot_parameter(self.ax_2theta, self.monitor.fit_data[x_data_type], self.monitor.fit_data['dois_theta_0'], 'Peak position (°)', x_label, 'PseudoVoigt #1', color='red')
        self._plot_parameter(self.ax_2theta, self.monitor.fit_data[x_data_type], self.monitor.fit_data['dois_theta_0_#2'], 'Peak position (°)', x_label, 'PseudoVoigt #2', color='red', marker='x')

        self._plot_parameter(self.ax_area, self.monitor.fit_data[x_data_type], self.monitor.fit_data['area'], 'Peak integrated area', x_label, 'PseudoVoigt #1', color='green')
        self._plot_parameter(self.ax_area, self.monitor.fit_data[x_data_type], self.monitor.fit_data['area_#2'], 'Peak integrated area', x_label, 'PseudoVoigt #2', color='green', marker='x')

        self._plot_parameter(self.ax_FWHM, self.monitor.fit_data[x_data_type], self.monitor.fit_data['fwhm'], 'FWHM (°)', x_label, 'PseudoVoigt #1', color='blue')
        self._plot_parameter(self.ax_FWHM, self.monitor.fit_data[x_data_type], self.monitor.fit_data['fwhm_#2'], 'FWHM (°)', x_label, 'PseudoVoigt #2', color='blue', marker='x')

    def _plot_parameter(self, ax, x, y, ylabel, xlabel, label=None, color=None, marker='o'):
        """
        Routine for plotting x and y on given axis (ax)
        """
        ax.plot(x, y, marker, label=label if label else '', color = color)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        if label:
            ax.legend()

    def select_folder(self):
        if self.folder_selected:
            self.monitor.data_frame = self.monitor.data_frame.iloc[0:0]
            self.monitor.fit_data = self.monitor.fit_data.iloc[0:0]
        counter.count = 0
        self.plot_with_temp = False
        self.selected_interval = None
        self.fit_interval = None
        
        self.folder_selected = False
        self.temp_mask_signal = False
        folder_path = QFileDialog.getExistingDirectory(self, 'Select the data folder to monitor', '', options=QFileDialog.Options()) # Selection of monitoring folder
        if folder_path:
            
            self.folder_selected = True
            self.monitor = FolderMonitor(folder_path=folder_path)
            self.monitor.new_data_signal.connect(self.handle_new_data)
            self.monitor.start()

        else:
            print('No folder selected. Exiting')
            

    def handle_new_data(self, new_data):
        self.plot_data = pd.concat([self.plot_data, new_data], ignore_index=True)
        
    def onselect(self, xmin, xmax):
        self.selected_interval = (xmin, xmax)
        #print(f'Selected Interval: {self.selected_interval}')
        self.update_graphs()
    # Reset button function #     
    def reset_interval(self):
        self.selected_interval = None
        self.update_graphs()
    # Peak fit interval selection routine # 
    def select_fit_interval(self):
        if not self.folder_selected:
            QMessageBox.warning(self, '','Please initialize the monitor!')
            pass
        else: 
            try:
                if len(self.plot_data['theta']) == 0:
                    print('No data available. Wait for the first measure!')
                else:
                    self.ax_2theta.clear()
                    self.ax_area.clear()
                    self.ax_FWHM.clear()
                    self.fit_interval=None
                    self.monitor.fit_data = self.monitor.fit_data.iloc[0:0] #Reset fitting data
                    self.fit_interval_window = FitWindow()
                    self.fit_interval_window.show()
            except AttributeError as e:
                print(f'Please, push the Refresh Button! Error: {e}')
                QMessageBox.warning(self, '','Please, push the Refresh Button!') 
            except Exception as e:
                print(f'Error: {e}')
     

    def save_data_frame(self):
        try:
            options = QFileDialog.Options()

        # Select appropriate DataFrame generator based on model and temperature
            if self.fit_interval_window.fit_model == 'PseudoVoigt':
                df = self._create_single_peak_dataframe()
            else:
                df = self._create_double_peak_dataframe()

            if df is not None:
                file_path, _ = QFileDialog.getSaveFileName(self, "Save fitting Data", "", "Excel Files (*.xlsx);;All Files (*)", options=options)
                if file_path:
                    df.to_csv(file_path)
        except AttributeError as e:
            print(f"No data available! Please, initialize the monitor! Error: {e}")
            QMessageBox.warning(self, '','Please initialize the monitor!')
        except Exception as e:
            print(f'Exception {e} encountered')

    def _create_single_peak_dataframe(self):
        if self.plot_with_temp:
            temp_label = 'Cryojet Temperature (K)' if self.monitor.kelvin_sginal else 'Blower Temperature (°C)'
            return pd.DataFrame({
            temp_label: self.monitor.fit_data['temp'],
            'Peak position (degree)': self.monitor.fit_data['dois_theta_0'],
            'Peak Integrated Area': self.monitor.fit_data['area'],
            'FWHM (degree)': self.monitor.fit_data['fwhm'],
            'R-squared (R²)': self.monitor.fit_data['R-squared']
        })
        else:
            return pd.DataFrame({
            'Measure': self.monitor.fit_data['file_index'],
            'Peak position (degree)': self.monitor.fit_data['dois_theta_0'],
            'Peak Integrated Area': self.monitor.fit_data['area'],
            'FWHM (degree)': self.monitor.fit_data['fwhm'],
            'R-squared (R²)': self.monitor.fit_data['R-squared']
        })

    def _create_double_peak_dataframe(self):
        if self.plot_with_temp:
            temp_label = 'Cryojet Temperature (K)' if self.monitor.kelvin_sginal else 'Blower Temperature (°C)'
            return pd.DataFrame({
            temp_label: self.monitor.fit_data['temp'],
            'Peak position #1 (degree)': self.monitor.fit_data['dois_theta_0'],
            'Peak Integrated Area #1': self.monitor.fit_data['area'],
            'FWHM (degree) #1': self.monitor.fit_data['fwhm'],
            'Peak Position #2 (degree)': self.monitor.fit_data['dois_theta_0_#2'],
            'Peak Integrated Area #2': self.monitor.fit_data['area_#2'],
            'FWHM #2 (degree)': self.monitor.fit_data['fwhm_#2'],
            'R-squared (R²)': self.monitor.fit_data['R-squared']
        })
        else:
            return pd.DataFrame({
            'Measure': self.monitor.fit_data['file_index'],
            'Peak position #1 (degree)': self.monitor.fit_data['dois_theta_0'],
            'Peak Integrated Area #1': self.monitor.fit_data['area'],
            'FWHM (degree) #1': self.monitor.fit_data['fwhm'],
            'Peak Position #2 (degree)': self.monitor.fit_data['dois_theta_0_#2'],
            'Peak Integrated Area #2': self.monitor.fit_data['area_#2'],
            'FWHM #2 (degree)': self.monitor.fit_data['fwhm_#2'],
            'R-squared (R²)': self.monitor.fit_data['R-squared']
        })

    def validate_temp(self, min_value, max_value):
        """
        Validates the temperature selected at the SpinBoxes. 
        """
        min_temp = min(self.monitor.data_frame['temp'], key=lambda x: abs(x-min_value))
        max_temp = min(self.monitor.data_frame['temp'], key=lambda x: abs(x-max_value))
        return min_temp, max_temp
    
    def apply_temp_mask(self):
        try:
            if self.plot_with_temp:
                min_temp, max_temp = self.validate_temp(self.min_temp_doubleSpinBox.value(), self.max_temp_doubleSpinBox_2.value())
                self.temp_mask = (self.monitor.data_frame['temp'] >= min_temp) & (self.monitor.data_frame['temp'] <= max_temp)
            else:
                self.temp_mask = (self.monitor.data_frame['file_index'] >= self.min_temp_doubleSpinBox.value()) & (self.monitor.data_frame['file_index'] <= self.max_temp_doubleSpinBox_2.value())
            self.temp_mask_signal = True
            self.update_graphs()
        except AttributeError as e:
            print(f"No data available! Please, initialize the monitor! Error: {e}")
            QMessageBox.warning(self, '','Please initialize the monitor!')
        except Exception as e:
            print(f'Exception {e} encountered')

    def measure_order_index(self, checked):
        if checked:
            self.temperature_checkbox.setCheckState(False)
            self.plot_with_temp = False
            self.min_filter_label.setText('<u>Minimum:</u>')
            self.max_filter_label.setText('<u>Maximum:</u>')
            self.update_graphs()
        else:
            self.temperature_checkbox.setCheckable(True)

    def temp_index(self, checked):
        try:
            if checked:

                if self.monitor.data_frame['temp'][0] != None:
                    self.XRD_measure_order_checkbox.setCheckState(False)
                    self.plot_with_temp = True
                    self.min_filter_label.setText('<u>Minimum Temperature:</u>')
                    self.max_filter_label.setText('<u>Maximum Temperature:</u>')
                    self.update_graphs()
                else:
                    print("This experiment doesn't make use of temperature!")
                    self.temperature_checkbox.setCheckState(False)
            else:
                pass
                
        except AttributeError as e:
            self.temperature_checkbox.setCheckState(False)
            print(f'Please initizalie the Monitor! Error {e}')
            QMessageBox.warning(self, '','Please initialize the monitor!')     
            
        
    def about(self):
        QMessageBox.about(
            self,
            "About Iguape",
            "<p>This is the Paineira Graphical User Interface</p>"
            "<p>- Its usage is resttricted to data acquired via in-situ experiments at Paineira. The software is under the GNU GPL-3.0 License.</p>"
            "<p>- There's a brief tutorial for first time users, which can be helpful, altough the program's operation is very simple"
            "<p>- Paineira Beamline</p>"
            "<p>- LNLS - CNPEM</p>",
        )


class Worker(QThread):
    """
    QThread Class for performing the Peak Fit. 
    """
    progress = pyqtSignal(int)
    finished = pyqtSignal(float) 
    error = pyqtSignal(str) # Changed to emit multiple arrays

    def __init__(self, interval):
        super().__init__()
        self.fit_interval = interval
        QApplication.setOverrideCursor(Qt.WaitCursor)

    def run(self):
        
        try:
            start = time.time()
            for i in range(len(win.plot_data['theta'])):
                if win.fit_interval_window.fit_model == 'PseudoVoigt':
                    fit = peak_fit(win.plot_data['theta'][i], win.plot_data['intensity'][i], self.fit_interval)
                    new_fit_data = pd.DataFrame({'dois_theta_0': [fit[0]], 'fwhm': [fit[1]], 'area': [fit[2]], 'temp': [win.plot_data['temp'][i]], 'file_index': [win.plot_data['file_index'][i]], 'R-squared': [fit[3]]})
                    win.monitor.fit_data = pd.concat([win.monitor.fit_data, new_fit_data], ignore_index=True)
                    progress_value = int((i + 1) / len(win.plot_data['theta']) * 100)
                    self.progress.emit(progress_value)  # Emit progress signal with percentage
                else:
                    fit = peak_fit_split_gaussian(win.plot_data['theta'][i], win.plot_data['intensity'][i], self.fit_interval, height = win.fit_interval_window.height, distance=win.fit_interval_window.distance)
                    new_fit_data = pd.DataFrame({'dois_theta_0': [fit[0][0]], 'dois_theta_0_#2': [fit[0][1]], 'fwhm': [fit[1][0]], 'fwhm_#2': [fit[1][1]], 'area': [fit[2][0]], 'area_#2': [fit[2][1]], 'temp': [win.plot_data['temp'][i]], 'file_index': [win.plot_data['file_index'][i]], 'R-squared': [fit[3]]})
                    win.monitor.fit_data =pd.concat([win.monitor.fit_data, new_fit_data], ignore_index=True)
               

                    progress_value = int((i + 1) / len(win.plot_data['theta']) * 100)
                    self.progress.emit(progress_value)  # Emit progress signal with percentage
            #self.finished.emit(win.monitor.fit_data['dois_theta_0'], win.monitor.fit_data['area'], win.monitor.fit_data['fwhm'])  # Emit finished signal with results
            finish = time.time()
            self.finished.emit(finish-start)

        except Exception as e:
            self.error.emit(f'Error during peak fitting: {str(e)}. Please select a new Fit Interval')
            print(f'Exception {e}. Please select a new Fit Interval')


class FitWindow(QDialog, Ui_pk_window):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.fit_interval= None
        self.text = None
        self.fit_model = 'PseudoVoigt'
        win.monitor.set_fit_model = "PseudoVoigt"
        self.distance = 25
        self.height = 1e+09
        self.setup_layout()

    def setup_layout(self):
        self.setWindowTitle('Peak Fit')
        self.pk_layout = QVBoxLayout()
        self.fig = Figure(figsize=(20,10), dpi=100)
        self.ax = self.fig.add_subplot(1,1,1)
        if win.plot_with_temp:
            self.ax.plot(win.plot_data['theta'][0], win.plot_data['intensity'][0],'o', markersize=3, label = 'XRD pattern ' + str(win.plot_data['temp'][0]) + '°C')
        
        else:
            self.ax.plot(win.plot_data['theta'][0], win.plot_data['intensity'][0], 'o', markersize=3, label = 'XRD pattern #' + str(win.plot_data['file_index'][0]))
        self.ax.set_xlabel("2θ (°)")
        self.ax.set_ylabel("Intensity (u.a.)")
        self.ax.legend(fontsize='small')
        self.canvas = FigureCanvas(self.fig)
        self.pk_layout.addWidget(self.canvas)
        self.pk_layout.addWidget(NavigationToolbar(self.canvas, self))
        self.pk_frame.setLayout(self.pk_layout)
        self.span = SpanSelector(self.ax, self.onselect, 'horizontal', useblit=True,
                                props=dict(alpha=0.3, facecolor='red', capstyle='round'))
        
        self.clear_plot_button.clicked.connect(self.clear_plot)
        self.pk_button.clicked.connect(self.fit)
        self.indexes = [0]
        self.shade = False


        if win.plot_with_temp:
            items_list = [str(item) + '°C' for item in win.plot_data['temp']]
            self.xrd_combo_box.addItems(items_list)
        else:
            items_list = [str(item) for item in win.plot_data['file_index']]
            self.xrd_combo_box.addItems(items_list)
        
        self.xrd_combo_box.activated[str].connect(self.onChanged_xrd_combo_box)
        self.pk_combo_box.activated[str].connect(self.onChanged_pk_combo_box)
        self.bgk_combo_box.activated[str].connect(self.onChanged_bkg_combo_box)
        self.preview_button.clicked.connect(self.preview)
        self.distance_spinBox.setReadOnly(True)
        self.distance_spinBox.valueChanged[int].connect(self.onChanged_distance_spinbox)
        self.height_spinBox.setReadOnly(True)
        self.height_spinBox.valueChanged[int].connect(self.onChanged_height_spinbox)

    def onChanged_xrd_combo_box(self, text):
        self.text = text
        if len(self.ax.lines) == 2:
            QMessageBox.warning(self, '','Warning! It is possible to display only two XRD patterns in this window! Please press the Clear Plot button and select up to 2 XRD patterns to be displayed.')
            pass
        else:
            i = self.xrd_combo_box.currentIndex()
            self.indexes.append(i)
            if win.plot_with_temp:
                self.ax.plot(win.plot_data['theta'][i], win.plot_data['intensity'][i], 'o', markersize=3, label = ('XRD pattern ' + text))
            else:
                self.ax.plot(win.plot_data['theta'][i], win.plot_data['intensity'][i], 'o', markersize=3, label = ('XRD pattern #' + text))
        
            self.ax.set_xlabel("2θ (°)")
            self.ax.set_ylabel("Intensity (u.a.)")
            self.ax.legend(fontsize='small')
            self.canvas.draw()
    
    def onChanged_pk_combo_box(self, text):
        if text == 'PseudoVoigt Model':
            self.fit_model = 'PseudoVoigt'
            win.monitor.set_fit_model = 'PseudoVoigt'
            self.distance_spinBox.setReadOnly(True)
            self.height_spinBox.setReadOnly(True)
        else:
            self.fit_model = '2x PseudoVoigt(SPV)'
            win.monitor.set_fit_model = '2x PseudoVoigt(SPV)'
            self.distance_spinBox.setReadOnly(False)
            self.height_spinBox.setReadOnly(False)
    def onChanged_bkg_combo_box(self, text):
        if text == 'Linear Model': 
            self.bkg_model = 'Linear'
        else:
            self.bkg_model = 'Spline'

    def onselect(self, xmin, xmax):
        if self.shade:
            self.shade.remove()
        self.fit_interval = [xmin, xmax]
        self.interval_label.setText(f'[{xmin:.3f}, {xmax:.3f}]')
        self.shade = self.ax.axvspan(self.fit_interval[0], self.fit_interval[1], color='grey', alpha=0.5, label='Selected Fitting Interval')
        self.canvas.draw()
        
    def onChanged_distance_spinbox(self, value):
        self.distance = value

    def onChanged_height_spinbox(self, value):
        self.height = value*(1e+09)
        

    def clear_plot(self):
        self.ax.clear()
        self.canvas.draw()
        self.indexes.clear()

    def preview(self):
        """
        Returns a Preview of the Peak Fitting for the selected Model and 2theta Interval.
        """
        if len(self.ax.lines) > 2:
            while len(self.ax.lines) > 2:
                self.ax.lines[len(self.ax.lines)-1].remove()
        if self.fit_model == "PseudoVoigt":
            for i in range(len(self.indexes)):
                data = peak_fit(win.plot_data['theta'][self.indexes[i]], win.plot_data['intensity'][self.indexes[i]], self.fit_interval)
                if win.plot_with_temp:
                    self.ax.plot(data[6], data[4].best_fit, '--', label = f'Best Fit - {win.plot_data['temp'][self.indexes[i]]} °C')
                    self.ax.plot(data[6], data[5]['bkg_'], '-', label = f'Background - {win.plot_data['temp'][self.indexes[i]]} °C')
                else:
                    self.ax.plot(data[6], data[4].best_fit, '--', label = f'Best Fit - #{win.plot_data['file_index'][self.indexes[i]]}')
                    self.ax.plot(data[6], data[5]['bkg_'], '-', label = f'Background - #{win.plot_data['file_index'][self.indexes[i]]}')
                self.ax.legend(fontsize='small')
                self.canvas.draw()
        else:
            for i in range(len(self.indexes)):
                try:
                    data = peak_fit_split_gaussian(win.plot_data['theta'][self.indexes[i]], win.plot_data['intensity'][self.indexes[i]], self.fit_interval, height = self.height, distance=self.distance)
                    if win.plot_with_temp:
                        self.ax.plot(data[6], data[4].best_fit, '--', label = f'Best Fit - {win.plot_data['temp'][self.indexes[i]]} °C')
                        self.ax.plot(data[6], data[5]['bkg_'], '-', label = f'Background - {win.plot_data['temp'][self.indexes[i]]} °C')
                    else:
                        self.ax.plot(data[6], data[4].best_fit, '--', label = f'Best Fit - #{win.plot_data['file_index'][self.indexes[i]]}')
                        self.ax.plot(data[6], data[5]['bkg_'], '-', label = f'Background - #{win.plot_data['file_index'][self.indexes[i]]}')
                    self.ax.legend(fontsize='small')
                    self.canvas.draw()
                except UnboundLocalError as e:
                    QMessageBox.warning(self, '', 'The value given for distance and/or height for peak search are out of bounds, i.e., it was not possible to find two peaks mtaching the given parameters! Please, try again with different values for distance and height!')

    def fit(self):
        win.monitor.set_fit_interval(self.fit_interval)
        win.monitor.set_distance(self.distance)
        win.monitor.set_height(self.height)
        win.fit_interval = self.fit_interval
        win.fit_interval_window.close()

        self.progress_dialog = QProgressDialog("Fitting peaks...", "", 0, 100, self)
        self.progress_dialog.setWindowModality(Qt.WindowModal)
        self.progress_dialog.setAutoClose(True)
        self.progress_dialog.show()
        self.progress_dialog.setCancelButton(None)

        # Start the worker thread for peak fitting
        self.worker = Worker(self.fit_interval)
        self.worker.progress.connect(self.update_progress)
        self.worker.finished.connect(self.peak_fitting_finished)
        self.worker.error.connect(self.peak_fitting_error)
        self.worker.start()

    def update_progress(self, value):
        self.progress_dialog.setValue(value)

    def peak_fitting_finished(self, time):
        self.progress_dialog.setValue(100)
        QMessageBox.information(self, "Peak Fitting", f"Peak fitting completed successfully! Elapsed time: {int(time)}s")
        win.update_graphs()
        
        self.close()

    def peak_fitting_error(self, error_message):
        self.progress_dialog.cancel()
        QMessageBox.warning(self, "Peak Fitting Error", error_message)
        self.show()

        

        


if __name__ == "__main__":
    app = QApplication(sys.argv)
    win = Window()
    win.show()
    sys.exit(app.exec())



#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information

"""
This is Monitor Class. It was built to track and read a given Folder for new XRD Data. It's dependent on the iguape_fileslist.txt text file.
It was built to work only for Paineira XRD Data, but it can easily be adjusted for other situations.
"""

import time, os, math
import numpy as np
import lmfit as lm
from lmfit.models import PseudoVoigtModel, LinearModel
import pandas as pd
from scipy.signal import find_peaks
from PyQt5.QtCore import QThread, pyqtSignal

# --- Monitor - Reading a '.txt' file for new data --- #
class FolderMonitor(QThread):
	new_data_signal = pyqtSignal(pd.DataFrame)
	def __init__(self, folder_path, fit_interval=None):
		super().__init__()
		self.folder_path = folder_path
		self.fit_interval = fit_interval
		self.fit_model = 'PseudoVoigt'
		self.kelvin_sginal = False
		self.processed_files = set()  # Tracking the processed files
		self.data_frame = pd.DataFrame(columns = ['theta', 'intensity', 'temp', 'max', 'file_index'])
		self.fit_data = pd.DataFrame(columns=['dois_theta_0', 'fwhm', 'area', 'temp', 'file_index', 'R-squared'])

	def run(self):
		reading_status = 1
		i = 0
		print(f'Monitoring folder: {self.folder_path}')
		print('Waiting for XRD data! Please, wait')
		while reading_status == 1:
			while True:
				try:
					with open(os.path.join(self.folder_path,'iguape_filelist.txt'),"r") as file:
						lines = file.read().splitlines()
						line = lines[i+1]
						data = data_read(os.path.join(self.folder_path,line))
						self.kelvin_sginal = data[3]
						file_index = counter()
						new_data = pd.DataFrame({'theta': [data[0]], 'intensity': [data[1]], 'temp': [data[2]], 'max': [data[1].max()], 'file_index': [file_index]})
						self.data_frame = pd.concat([self.data_frame, new_data], ignore_index=True)
						self.new_data_signal.emit(new_data)
						print(f"New data created at: {self.folder_path}. File name: {lines[i+1]}")
						if self.fit_interval:
							if self.fit_model == 'PseudoVoigt':
								fit = peak_fit(data[0], data[1], self.fit_interval)
								new_fit_data = pd.DataFrame({'dois_theta_0': [fit[0]], 'fwhm': [fit[1]], 'area': [fit[2]], 'temp': [data[2]], 'file_index': [file_index], 'R-squared': [fit[3]]})
								self.fit_data = pd.concat([self.fit_data, new_fit_data], ignore_index=True)
							else:
								fit = peak_fit_split_gaussian(data[0], data[1], self.fit_interval, height=self.height, distance = self.distance)
								new_fit_data = pd.DataFrame({'dois_theta_0': [fit[0]][0], 'fwhm': [fit[1][0]], 'area': [fit[2][0]], 'temp': [data[2]], 'file_index': [file_index], 'R-squared': [fit[3]]})
								self.fit_data = pd.concat([self.fit_data, new_fit_data], ignore_index=True)
								self.fit_data.insert(1,'dois_theta_0_#2', [fit[0][1]])
								self.fit_data.insert(3, 'fwhm_#2', [fit[1][1]])
								self.fit_data.insert(5, 'area_#2', [fit[2][1]])

						reading_status = int(lines[i+2])
					break
				except Exception as e:
					#print(f'Exception: {e}')
					time.sleep(0.1)
			
			i+=2
		

	def set_fit_interval(self, interval):
		self.fit_interval = interval
	def set_fit_model(self, model):
		self.fit_model= model
	def set_distance(self, distance):
		self.distance = distance
	def set_height(self, height):
		self.height = height
# --- Defining the functions for data reading and peak fitting --- #
def data_read(path):
	done = False
	while not done:
		time.sleep(0.1)
		try:
			dados = pd.read_csv(path, sep=',')
			x = np.array(dados.get('2theta (degree)'))
			y = np.array(dados.get('Intensity'))
			file_name = path.split(sep='/')[len(path.split(sep='/'))-1]
			temp = None
			kelvin_signal = None
			for i in file_name.split(sep='_'):
				if 'Celsius' in i: 
					temp = float(i.split(sep='Celsius')[0]) #Getting the temperature
				elif 'Kelvin' in i:
					temp = float(i.split(sep='Kelvin')[0])
					kelvin_signal = True
			done = True
			return x,y,temp, kelvin_signal
		except pd.errors.EmptyDataError:
			print(f"Warning: Empty file encountered: {path}. Trying to read the data again!")
			#return None
		except Exception as e:
			print(f"An error occurred while reading data: {e}. Trying to read the data again!")
			#return None

# --- Defining the storaging lists --- #		


def peak_fit(theta, intensity, interval, bkg = 'Linear'):
	done = False
	while not done:
		#time.sleep(0.5)
		try:
			theta_fit = []
			intensity_fit = []
  
  # Slicing the data for the selected peak fitting interval #
			for i in range(len(theta)):
				if theta[i] > interval[0] and theta[i] < interval[1]: 
					theta_fit.append(theta[i])
					intensity_fit.append(intensity[i])
			theta_fit=np.array(theta_fit)
			intensity_fit=np.array(intensity_fit)
  # Building the Voigt model with lmfit #
			
			mod = PseudoVoigtModel(nan_policy='omit')
			pars = mod.guess(data= intensity_fit, x = theta_fit)
			background = LinearModel(prefix='bkg_')
			pars.update(background.guess(data=intensity_fit, x = theta_fit))
			mod += background
			
			out = mod.fit(intensity_fit, pars, x=theta_fit) # Fitting the data to the Voigt model #
			comps = out.eval_components(x=theta_fit)
  # Getting the parameters from the optimal fit #, bkg= self.bkg_model
			
			dois_theta_0 = out.params['center']*1
			fwhm = out.params['fwhm']*1
			area = out.params['amplitude']*1
			r_squared = out.rsquared

			done = True
			return dois_theta_0, fwhm, area, r_squared, out, comps, theta_fit
		except ValueError or TypeError as e:
			print(f'Fitting error, please wait: {e}! Please select a new fitting interval')
			done = True
			pass

def pseudo_voigt(x, amplitude, center, sigma, eta):
    sigma_g = sigma/math.sqrt(2*math.log(2))
    gaussian = (amplitude/(sigma_g*math.sqrt(2*math.pi)))*np.exp(-(x-center)**2/(2*sigma_g** 2))
    lorentzian = ((amplitude/math.pi)*sigma)/((x - center)**2 + sigma**2)
    return eta*lorentzian + (1 - eta)*gaussian

def split_pseudo_voigt(x, amp1, cen1, sigma1, eta1, amp2, cen2, sigma2, eta2):
    return (pseudo_voigt(x, amplitude=amp1, center=cen1, sigma=sigma1, eta=eta1) +
            pseudo_voigt(x, amplitude=amp2, center=cen2, sigma=sigma2, eta=eta2))

def peak_fit_split_gaussian(theta, intensity, interval, bkg = 'Linear', height=1e+09, distance = 35):
	done = False
	while not done:
		#time.sleep(0.5)
		try:
			theta_fit = []
			intensity_fit = []
  
  # Slicing the data for the selected peak fitting interval #
			for i in range(len(theta)):
				if theta[i] > interval[0] and theta[i] < interval[1]: 
					theta_fit.append(theta[i])
					intensity_fit.append(intensity[i])
			theta_fit=np.array(theta_fit)
			intensity_fit=np.array(intensity_fit)
  # Building the Voigt model with lmfit #
			
			peaks, properties = find_peaks(intensity_fit, height=height, distance=distance)
			if len(peaks) >= 2:
	# Sort peaks by prominence and pick the top two
				sorted_indices = np.argsort(properties['peak_heights'])[-2:]
				peak_positions = theta_fit[peaks][sorted_indices]
				peak_heights = properties['peak_heights'][sorted_indices]
				if peak_positions[0] > peak_positions[1]:
					amp2, cen2 = peak_heights[0], peak_positions[0]
					amp1, cen1 = peak_heights[1], peak_positions[1]
				else:
					amp1, cen1 = peak_heights[0], peak_positions[0]
					amp2, cen2 = peak_heights[1], peak_positions[1]

	# Estimate sigma using the width of the peaks at half height
				sigma1 = 0.1/2.355
				sigma2 = 0.1/2.355
				
				

			model = lm.Model(split_pseudo_voigt)
			pars = model.make_params(amp1=amp1, cen1=cen1, sigma1=sigma1, eta1=0.5, amp2=amp2, cen2=cen2, sigma2=sigma2, eta2=0.5)
			pars['eta1'].min, pars['eta1'].max = 0, 1
			pars['eta2'].min, pars['eta2'].max = 0, 1
			background = LinearModel(prefix='bkg_')
			pars.update(background.guess(data=intensity_fit, x = theta_fit))
			model += background		

			out = model.fit(intensity_fit, pars, x=theta_fit) # Fitting the data to the Voigt model #
			comps = out.eval_components(x=theta_fit)
			
  # Getting the parameters from the optimal fit #, bkg= self.bkg_model
			
			dois_theta_0 = [out.params['cen1']*1, out.params['cen2']*1]
			fwhm = [2.0*out.params['sigma1'], 2.0*out.params['sigma2']]
			area = [out.params['amp1']*1, out.params['amp2']*1]
			r_squared = out.rsquared
			done = True
			return dois_theta_0, fwhm, area, r_squared, out, comps, theta_fit
		except ValueError or TypeError as e:
			print(f'Fitting error, please wait: {e}! Please select a new fitting interval')
			done = True
			pass




# --- A counter function to index the created curves --- #
def counter():
	counter.count += 1
	return counter.count
	
counter.count = 0



#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information



#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information

"""
Main GUI python script. This GUI was built with PyQt5. The .ui file can be found inside the GUI folder.
"""

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(852, 605)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("Icons/Logo_IGUAPE.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setStyleSheet("")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.reset_button = QtWidgets.QPushButton(self.centralwidget)
        self.reset_button.setMinimumSize(QtCore.QSize(0, 0))
        self.reset_button.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.reset_button.setObjectName("reset_button")
        self.gridLayout.addWidget(self.reset_button, 2, 3, 1, 1)
        self.offset_label = QtWidgets.QLabel(self.centralwidget)
        self.offset_label.setObjectName("offset_label")
        self.gridLayout.addWidget(self.offset_label, 2, 0, 1, 1)
        self.min_temp_doubleSpinBox = QtWidgets.QDoubleSpinBox(self.centralwidget)
        self.min_temp_doubleSpinBox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.min_temp_doubleSpinBox.setDecimals(0)
        self.min_temp_doubleSpinBox.setObjectName("min_temp_doubleSpinBox")
        self.gridLayout.addWidget(self.min_temp_doubleSpinBox, 4, 1, 1, 1)
        self.offset_slider = QtWidgets.QSlider(self.centralwidget)
        self.offset_slider.setMaximumSize(QtCore.QSize(500, 16777215))
        self.offset_slider.setOrientation(QtCore.Qt.Horizontal)
        self.offset_slider.setObjectName("offset_slider")
        self.gridLayout.addWidget(self.offset_slider, 2, 1, 1, 1)
        self.XRD_measure_order_checkbox = QtWidgets.QCheckBox(self.centralwidget)
        self.XRD_measure_order_checkbox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.XRD_measure_order_checkbox.setCheckable(True)
        self.XRD_measure_order_checkbox.setChecked(True)
        self.XRD_measure_order_checkbox.setTristate(False)
        self.XRD_measure_order_checkbox.setObjectName("XRD_measure_order_checkbox")
        self.gridLayout.addWidget(self.XRD_measure_order_checkbox, 3, 1, 1, 1)
        self.index_label = QtWidgets.QLabel(self.centralwidget)
        self.index_label.setObjectName("index_label")
        self.gridLayout.addWidget(self.index_label, 3, 0, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.XRD_data_tab = QtWidgets.QWidget()
        self.XRD_data_tab.setObjectName("XRD_data_tab")
        self.tabWidget.addTab(self.XRD_data_tab, "")
        self.peak_fit_tab = QtWidgets.QWidget()
        self.peak_fit_tab.setObjectName("peak_fit_tab")
        self.tabWidget.addTab(self.peak_fit_tab, "")
        self.gridLayout.addWidget(self.tabWidget, 1, 0, 1, 6)
        self.save_peak_fit_data_button = QtWidgets.QPushButton(self.centralwidget)
        self.save_peak_fit_data_button.setMinimumSize(QtCore.QSize(0, 0))
        self.save_peak_fit_data_button.setMaximumSize(QtCore.QSize(16777215, 16777215))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("Icons/Save_Icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.save_peak_fit_data_button.setIcon(icon1)
        self.save_peak_fit_data_button.setObjectName("save_peak_fit_data_button")
        self.gridLayout.addWidget(self.save_peak_fit_data_button, 3, 4, 1, 2)
        self.min_filter_label = QtWidgets.QLabel(self.centralwidget)
        self.min_filter_label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.min_filter_label.setObjectName("min_filter_label")
        self.gridLayout.addWidget(self.min_filter_label, 4, 0, 1, 1)
        self.CNPEM_logo = QtWidgets.QLabel(self.centralwidget)
        self.CNPEM_logo.setMaximumSize(QtCore.QSize(100, 50))
        self.CNPEM_logo.setText("")
        self.CNPEM_logo.setPixmap(QtGui.QPixmap("Icons/CNPEM-logo-colorido-2023_resize.png"))
        self.CNPEM_logo.setObjectName("CNPEM_logo")
        self.gridLayout.addWidget(self.CNPEM_logo, 0, 0, 1, 1)
        self.max_filter_label = QtWidgets.QLabel(self.centralwidget)
        self.max_filter_label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.max_filter_label.setObjectName("max_filter_label")
        self.gridLayout.addWidget(self.max_filter_label, 4, 2, 1, 1)
        self.paineir_logo = QtWidgets.QLabel(self.centralwidget)
        self.paineir_logo.setMaximumSize(QtCore.QSize(70, 100))
        self.paineir_logo.setText("")
        self.paineir_logo.setPixmap(QtGui.QPixmap("Icons/Paineira_resize.png"))
        self.paineir_logo.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.paineir_logo.setObjectName("paineir_logo")
        self.gridLayout.addWidget(self.paineir_logo, 0, 5, 1, 1)
        self.refresh_button = QtWidgets.QPushButton(self.centralwidget)
        self.refresh_button.setMaximumSize(QtCore.QSize(16777215, 16777215))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("Icons/Refresh_Icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refresh_button.setIcon(icon2)
        self.refresh_button.setObjectName("refresh_button")
        self.gridLayout.addWidget(self.refresh_button, 2, 2, 1, 1)
        self.peak_fit_button = QtWidgets.QPushButton(self.centralwidget)
        self.peak_fit_button.setMaximumSize(QtCore.QSize(16777215, 16777215))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("Icons/peak_fit_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.peak_fit_button.setIcon(icon3)
        self.peak_fit_button.setObjectName("peak_fit_button")
        self.gridLayout.addWidget(self.peak_fit_button, 2, 4, 1, 2)
        self.LNLS_logo = QtWidgets.QLabel(self.centralwidget)
        self.LNLS_logo.setMaximumSize(QtCore.QSize(16777215, 50))
        self.LNLS_logo.setText("")
        self.LNLS_logo.setPixmap(QtGui.QPixmap("Icons/LNLS_logo_resize.png"))
        self.LNLS_logo.setAlignment(QtCore.Qt.AlignCenter)
        self.LNLS_logo.setObjectName("LNLS_logo")
        self.gridLayout.addWidget(self.LNLS_logo, 0, 4, 1, 1)
        self.max_temp_doubleSpinBox_2 = QtWidgets.QDoubleSpinBox(self.centralwidget)
        self.max_temp_doubleSpinBox_2.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.max_temp_doubleSpinBox_2.setFrame(True)
        self.max_temp_doubleSpinBox_2.setDecimals(0)
        self.max_temp_doubleSpinBox_2.setObjectName("max_temp_doubleSpinBox_2")
        self.gridLayout.addWidget(self.max_temp_doubleSpinBox_2, 4, 3, 1, 1)
        self.temperature_checkbox = QtWidgets.QCheckBox(self.centralwidget)
        self.temperature_checkbox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.temperature_checkbox.setObjectName("temperature_checkbox")
        self.gridLayout.addWidget(self.temperature_checkbox, 3, 2, 1, 1)
        self.filter_button = QtWidgets.QPushButton(self.centralwidget)
        self.filter_button.setMaximumSize(QtCore.QSize(16777215, 16777215))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap("Icons/Filter_Icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.filter_button.setIcon(icon4)
        self.filter_button.setObjectName("filter_button")
        self.gridLayout.addWidget(self.filter_button, 4, 4, 1, 2)
        self.title_label = QtWidgets.QLabel(self.centralwidget)
        self.title_label.setMinimumSize(QtCore.QSize(0, 0))
        self.title_label.setMaximumSize(QtCore.QSize(16777215, 50))
        self.title_label.setFrameShape(QtWidgets.QFrame.HLine)
        self.title_label.setFrameShadow(QtWidgets.QFrame.Raised)
        self.title_label.setLineWidth(1)
        self.title_label.setAlignment(QtCore.Qt.AlignCenter)
        self.title_label.setObjectName("title_label")
        self.gridLayout.addWidget(self.title_label, 0, 1, 1, 3)
        self.verticalLayout.addLayout(self.gridLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 852, 20))
        self.menubar.setObjectName("menubar")
        self.menuFolder = QtWidgets.QMenu(self.menubar)
        self.menuFolder.setObjectName("menuFolder")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen_New_Folder = QtWidgets.QAction(MainWindow)
        self.actionOpen_New_Folder.setObjectName("actionOpen_New_Folder")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionHelp = QtWidgets.QAction(MainWindow)
        self.actionHelp.setObjectName("actionHelp")
        self.menuFolder.addAction(self.actionOpen_New_Folder)
        self.menuFolder.addSeparator()
        self.menuFolder.addAction(self.actionAbout)
        self.menuFolder.addAction(self.actionHelp)
        self.menubar.addAction(self.menuFolder.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Iguape"))
        self.reset_button.setText(_translate("MainWindow", "Reset 2θ"))
        self.offset_label.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; text-decoration: underline;\">XRD patterns offset:</span></p></body></html>"))
        self.XRD_measure_order_checkbox.setText(_translate("MainWindow", "XRD measure order"))
        self.index_label.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; text-decoration: underline;\">Select XDR patterns indexing:</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.XRD_data_tab), _translate("MainWindow", "XRD Data"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.peak_fit_tab), _translate("MainWindow", "Peak Fitting"))
        self.save_peak_fit_data_button.setText(_translate("MainWindow", "Save Peak Fit Data"))
        self.min_filter_label.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; text-decoration: underline;\">Minimum:</span></p></body></html>"))
        self.max_filter_label.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; text-decoration: underline;\">Maximum:</span></p></body></html>"))
        self.refresh_button.setText(_translate("MainWindow", "Refresh"))
        self.peak_fit_button.setText(_translate("MainWindow", "Peak Fit"))
        self.temperature_checkbox.setText(_translate("MainWindow", "Blower Temperature"))
        self.filter_button.setText(_translate("MainWindow", "Apply Filter "))
        self.title_label.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\"><span style=\" font-weight:600;\">IGUAPE</span></p><p align=\"center\"><span style=\" font-size:7pt; font-weight:600;\">(PAINEIRA GRAPHICAL USER INTERFACE)</span></p></body></html>"))
        self.menuFolder.setTitle(_translate("MainWindow", "Folder"))
        self.actionOpen_New_Folder.setText(_translate("MainWindow", "Open New Folder"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionHelp.setText(_translate("MainWindow", "Help"))



#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information

"""
Execution script. It goes to the directory where Iguape is installed and it executes the program (iguape.py)
"""

import subprocess
import os


def main():
    iguape_path = os.path.join(os.path.dirname(__file__), "iguape.py")
    iguape_dir = os.path.dirname(__file__)
    os.chdir(iguape_dir)
    subprocess.run(["python3", 'iguape.py'])

if __name__=='__main__':
    main()



#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information

"""
Main GUI python script. This GUI was built with PyQt5. The .ui file can be found inside the GUI folder.
"""

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_pk_window(object):
    def setupUi(self, pk_window):
        pk_window.setObjectName("pk_window")
        pk_window.resize(829, 594)
        self.verticalLayout = QtWidgets.QVBoxLayout(pk_window)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.theta_interval_label = QtWidgets.QLabel(pk_window)
        self.theta_interval_label.setObjectName("theta_interval_label")
        self.gridLayout.addWidget(self.theta_interval_label, 9, 0, 1, 1)
        self.preview_button = QtWidgets.QPushButton(pk_window)
        self.preview_button.setObjectName("preview_button")
        self.gridLayout.addWidget(self.preview_button, 3, 3, 1, 1)
        self.pk_button = QtWidgets.QPushButton(pk_window)
        self.pk_button.setObjectName("pk_button")
        self.gridLayout.addWidget(self.pk_button, 4, 3, 1, 1)
        self.pk_combo_box = QtWidgets.QComboBox(pk_window)
        self.pk_combo_box.setObjectName("pk_combo_box")
        self.pk_combo_box.addItem("")
        self.pk_combo_box.addItem("")
        self.gridLayout.addWidget(self.pk_combo_box, 6, 1, 1, 1)
        self.xrd_label = QtWidgets.QLabel(pk_window)
        self.xrd_label.setObjectName("xrd_label")
        self.gridLayout.addWidget(self.xrd_label, 3, 0, 1, 1)
        self.distance_label = QtWidgets.QLabel(pk_window)
        self.distance_label.setObjectName("distance_label")
        self.gridLayout.addWidget(self.distance_label, 6, 2, 1, 1)
        self.distance_spinBox = QtWidgets.QSpinBox(pk_window)
        self.distance_spinBox.setWrapping(False)
        self.distance_spinBox.setReadOnly(False)
        self.distance_spinBox.setMinimum(25)
        self.distance_spinBox.setObjectName("distance_spinBox")
        self.gridLayout.addWidget(self.distance_spinBox, 6, 3, 1, 1)
        self.interval_label = QtWidgets.QLabel(pk_window)
        self.interval_label.setText("")
        self.interval_label.setObjectName("interval_label")
        self.gridLayout.addWidget(self.interval_label, 9, 1, 1, 1)
        self.pk_label = QtWidgets.QLabel(pk_window)
        self.pk_label.setObjectName("pk_label")
        self.gridLayout.addWidget(self.pk_label, 6, 0, 1, 1)
        self.pk_frame = QtWidgets.QFrame(pk_window)
        self.pk_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.pk_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.pk_frame.setObjectName("pk_frame")
        self.gridLayout.addWidget(self.pk_frame, 1, 0, 2, 4)
        self.xrd_combo_box = QtWidgets.QComboBox(pk_window)
        self.xrd_combo_box.setObjectName("xrd_combo_box")
        self.gridLayout.addWidget(self.xrd_combo_box, 3, 1, 1, 1)
        self.bgk_label = QtWidgets.QLabel(pk_window)
        self.bgk_label.setObjectName("bgk_label")
        self.gridLayout.addWidget(self.bgk_label, 4, 0, 1, 1)
        self.bgk_combo_box = QtWidgets.QComboBox(pk_window)
        self.bgk_combo_box.setObjectName("bgk_combo_box")
        self.bgk_combo_box.addItem("")
        self.bgk_combo_box.addItem("")
        self.gridLayout.addWidget(self.bgk_combo_box, 4, 1, 1, 1)
        self.clear_plot_button = QtWidgets.QPushButton(pk_window)
        self.clear_plot_button.setObjectName("clear_plot_button")
        self.gridLayout.addWidget(self.clear_plot_button, 3, 2, 1, 1)
        self.height_label = QtWidgets.QLabel(pk_window)
        self.height_label.setObjectName("height_label")
        self.gridLayout.addWidget(self.height_label, 9, 2, 1, 1)
        self.height_spinBox = QtWidgets.QSpinBox(pk_window)
        self.height_spinBox.setMinimum(1)
        self.height_spinBox.setObjectName("height_spinBox")
        self.gridLayout.addWidget(self.height_spinBox, 9, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)

        self.retranslateUi(pk_window)
        QtCore.QMetaObject.connectSlotsByName(pk_window)

    def retranslateUi(self, pk_window):
        _translate = QtCore.QCoreApplication.translate
        pk_window.setWindowTitle(_translate("pk_window", "Dialog"))
        self.theta_interval_label.setText(_translate("pk_window", "<html><head/><body><p><span style=\" text-decoration: underline;\">2θ Interval Selected:</span></p></body></html>"))
        self.preview_button.setText(_translate("pk_window", "Preview"))
        self.pk_button.setText(_translate("pk_window", "Fit"))
        self.pk_combo_box.setItemText(0, _translate("pk_window", "PseudoVoigt Model"))
        self.pk_combo_box.setItemText(1, _translate("pk_window", "Split PseudoVoigt Model - 2x PseudoVoigt"))
        self.xrd_label.setText(_translate("pk_window", "<html><head/><body><p><span style=\" text-decoration: underline;\">XRD pattern:</span></p></body></html>"))
        self.distance_label.setText(_translate("pk_window", "<html><head/><body><p><span style=\" text-decoration: underline;\">Distance:</span></p></body></html>"))
        self.pk_label.setText(_translate("pk_window", "<html><head/><body><p><span style=\" text-decoration: underline;\">Peak Profile:</span></p></body></html>"))
        self.bgk_label.setText(_translate("pk_window", "<html><head/><body><p><span style=\" text-decoration: underline;\">Background Model:</span></p></body></html>"))
        self.bgk_combo_box.setItemText(0, _translate("pk_window", "Linear Model"))
        self.bgk_combo_box.setItemText(1, _translate("pk_window", "Spline Model"))
        self.clear_plot_button.setText(_translate("pk_window", "Clear Plot"))
        self.height_label.setText(_translate("pk_window", "<html><head/><body><p><span style=\" text-decoration: underline;\">Height (x1e+09 a.u.):</span></p></body></html>"))



#This is part of the source code for the Paineira Graphical User Interface - Iguape
#The code is distributed under the GNU GPL-3.0 License. Please refer to the main page (https://github.com/cnpem/iguape) for more information

"""
This is the main script for the excution of the Paineira Graphical User Interface, a GUI for visualization and data processing during in situ experiments at Paineira.
In this script, both GUIs used by the program are called and all of the backend functions and processes are defined. 
"""

__version__ = '1.0.2'
license  = 'GNU GPL-3.0 License'
